import 'dart:ui';
import 'package:flutter/material.dart';
import 'package:nui_core/data/nui_ent.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_theme/nui_theme.dart';
import 'nui_text_theme.dart';

class NUIColorTheme extends NUIEnt<NUIColorTheme> with NUIColorCodes{
  NUIColorTheme.empty();

  Map<String, String> _toolbar;
  Map<String, String> _toolbarShadow;
  Map<String, String> _statusBar;
  Map<String, String> _background;
  Map<String, String> _primary;
  Map<String, String> _primaryVariant;
  Map<String, String> _secondary;
  Map<String, String> _secondaryVariant;
  Map<String, String> _card;
  Map<String, String> _cardShadow;
  Map<String, String> _line;
  Map<String, String> _error;
  Map<String, String> _button;
  Map<String, String> _buttonDisabled;
  Map<String, String> _buttonShadow;
  Map<String, String> _icon;
  Map<String, String> _iconHighlight;

  Map<String, String> _onBackground; //Color on background
  Map<String, String> _onCard; //Color on card
  Map<String, String> _onPrimary; //Color on primary
  Map<String, String> _onSecondary; //Color on secondary
  Map<String, String> _onError; //Color on error

  Map<String, NUIColorGradient> _gradient;
  NUITextTheme _textTheme; //Text theme
  bool _isDark; //Brightness

  Color toolbar({String v}) => _getColorFromMap(_toolbar, v);
  Color toolbarShadow({String v}) => _getColorFromMap(_toolbarShadow, v);
  Color statusBar({String v}) => _getColorFromMap(_statusBar, v);
  Color background({String v}) => _getColorFromMap(_background, v);
  Color primary({String v}) => _getColorFromMap(_primary, v);
  Color primaryVariant({String v}) => _getColorFromMap(_primaryVariant, v);
  Color secondary({String v}) => _getColorFromMap(_secondary, v);
  Color secondaryVariant({String v}) => _getColorFromMap(_secondaryVariant, v);
  Color card({String v}) => _getColorFromMap(_card, v);
  Color cardShadow({String v}) => _getColorFromMap(_cardShadow, v);
  Color error({String v}) => _getColorFromMap(_error, v);
  Color button({String v}) => _getColorFromMap(_button, v);
  Color buttonDisabled({String v}) => _getColorFromMap(_buttonDisabled, v);
  Color buttonShadow({String v}) => _getColorFromMap(_buttonShadow, v);
  Color icon({String v}) => _getColorFromMap(_icon, v);
  Color iconHighlight({String v}) => _getColorFromMap(_iconHighlight, v);
  Color line({String v}) => _getColorFromMap(_line, v);

  Color onBackground({String v}) => _getColorFromMap(_onBackground, v);
  Color onPrimary({String v}) => _getColorFromMap(_onPrimary, v);
  Color onSecondary({String v}) => _getColorFromMap(_onSecondary, v);
  Color onError({String v}) => _getColorFromMap(_onError, v);
  Color onCard({String v}) => _getColorFromMap(_onCard, v);

  NUILinearGradient gradient({String v}) => _getGradientFromMap(_gradient, v);

  NUITextTheme get textTheme {
    _textTheme.setColorCodes(colorCodes);
    return _textTheme;
  }

  bool get isDark => _isDark ?? false;

  ColorScheme getColorScheme(){
    return ColorScheme(
        primary: primary(),
        primaryVariant: primaryVariant(),
        secondary: secondary(),
        secondaryVariant: secondaryVariant(),
        surface: card(),
        background: background(),
        error: error(),
        onPrimary: onPrimary(),
        onSecondary: onSecondary(),
        onSurface: onCard(),
        onBackground: onBackground(),
        onError: onError(),
        brightness: !isDark ? Brightness.light : Brightness.dark
    );
  }

  Color _getColorFromMap(Map<String, String> group, String variant){
    if(group == null || isNullOrEmpty(group.values.toList())){
      throwNUI("NUIColorTheme", "You have not configured color group");
    }

    if(isNullOrEmpty(variant)){
      final list = group.values.toList();
      final color = list[0];
      String targetColor = color;
      if(!isPossibleColor(color)){
        targetColor = hexColorFromCode(color) ?? color;
      }
      return HexColor(targetColor);
    }

    final color = group[variant];
    if(color == null){
      throwNUI("NUIColorTheme", "You have not configured a color for variant : $variant");
    }

    String targetColor = color;
    if(!isPossibleColor(color)){
      targetColor = hexColorFromCode(color) ?? color;
    }
    return HexColor(targetColor);
  }

  NUILinearGradient _getGradientFromMap(Map<String, NUIColorGradient> group, String variant){
    if(group == null || isNullOrEmpty(group.values.toList())){
      throwNUI("NUIColorTheme", "You have not configured color gradient group");
    }

    if(isNullOrEmpty(variant)){
      final list = group.values.toList();
      return list[0].getNUILinearGradient();
    }

    final colorStyle = group[variant];
    if(colorStyle == null){
      throwNUI("NUIColorTheme", "You have not configured a color gradient style for variant : $variant");
    }
    return colorStyle.getNUILinearGradient();
  }

  @override
  NUIEntMapper<NUIColorTheme> mapper() => NUIColorThemeEnt();
}

class NUIColorThemeEnt extends NUIEntMapper<NUIColorTheme>{
  @override
  String entityName() => "NUIColorTheme";

  @override
  List<NUIEntField> fields() {
    return [
      NUIEntField<NUIColorTheme>(name: "toolbar", type: NUIEntType.MAP, setter: (data, value) => data._toolbar = asMapOfType<String>(value), getter: (data) => data._toolbar),
      NUIEntField<NUIColorTheme>(name: "toolbarShadow", type: NUIEntType.MAP, setter: (data, value) => data._toolbarShadow = asMapOfType<String>(value), getter: (data) => data._toolbarShadow),
      NUIEntField<NUIColorTheme>(name: "statusBar", type: NUIEntType.MAP, setter: (data, value) => data._statusBar = asMapOfType<String>(value), getter: (data) => data._statusBar),
      NUIEntField<NUIColorTheme>(name: "background", type: NUIEntType.MAP, setter: (data, value) => data._background = asMapOfType<String>(value), getter: (data) => data._background),
      NUIEntField<NUIColorTheme>(name: "primary", type: NUIEntType.MAP, setter: (data, value) => data._primary = asMapOfType<String>(value), getter: (data) => data._primary),
      NUIEntField<NUIColorTheme>(name: "primaryVariant", type: NUIEntType.MAP, setter: (data, value) => data._primaryVariant = asMapOfType<String>(value), getter: (data) => data._primaryVariant),
      NUIEntField<NUIColorTheme>(name: "secondary", type: NUIEntType.MAP, setter: (data, value) => data._secondary = asMapOfType<String>(value), getter: (data) => data._secondary),
      NUIEntField<NUIColorTheme>(name: "secondaryVariant", type: NUIEntType.MAP, setter: (data, value) => data._secondaryVariant = asMapOfType<String>(value), getter: (data) => data._secondaryVariant),
      NUIEntField<NUIColorTheme>(name: "card", type: NUIEntType.MAP, setter: (data, value) => data._card = asMapOfType<String>(value), getter: (data) => data._card),
      NUIEntField<NUIColorTheme>(name: "cardShadow", type: NUIEntType.MAP, setter: (data, value) => data._cardShadow = asMapOfType<String>(value), getter: (data) => data._cardShadow),
      NUIEntField<NUIColorTheme>(name: "error", type: NUIEntType.MAP, setter: (data, value) => data._error = asMapOfType<String>(value), getter: (data) => data._error),
      NUIEntField<NUIColorTheme>(name: "button", type: NUIEntType.MAP, setter: (data, value) => data._button = asMapOfType<String>(value), getter: (data) => data._button),
      NUIEntField<NUIColorTheme>(name: "buttonDisabled", type: NUIEntType.MAP, setter: (data, value) => data._buttonDisabled = asMapOfType<String>(value), getter: (data) => data._buttonDisabled),
      NUIEntField<NUIColorTheme>(name: "buttonShadow", type: NUIEntType.MAP, setter: (data, value) => data._buttonShadow = asMapOfType<String>(value), getter: (data) => data._buttonShadow),
      NUIEntField<NUIColorTheme>(name: "icon", type: NUIEntType.MAP, setter: (data, value) => data._icon = asMapOfType<String>(value), getter: (data) => data._icon),
      NUIEntField<NUIColorTheme>(name: "iconHighlight", type: NUIEntType.MAP, setter: (data, value) => data._iconHighlight = asMapOfType<String>(value), getter: (data) => data._iconHighlight),
      NUIEntField<NUIColorTheme>(name: "line", type: NUIEntType.MAP, setter: (data, value) => data._line = asMapOfType<String>(value), getter: (data) => data._line),

      NUIEntField<NUIColorTheme>(name: "onBackground", type: NUIEntType.MAP, setter: (data, value) => data._onBackground = asMapOfType<String>(value), getter: (data) => data._onBackground),
      NUIEntField<NUIColorTheme>(name: "onPrimary", type: NUIEntType.MAP, setter: (data, value) => data._onPrimary = asMapOfType<String>(value), getter: (data) => data._onPrimary),
      NUIEntField<NUIColorTheme>(name: "onSecondary", type: NUIEntType.MAP, setter: (data, value) => data._onSecondary = asMapOfType<String>(value), getter: (data) => data._onSecondary),
      NUIEntField<NUIColorTheme>(name: "onError", type: NUIEntType.MAP, setter: (data, value) => data._onError = asMapOfType<String>(value), getter: (data) => data._onError),
      NUIEntField<NUIColorTheme>(name: "onCard", type: NUIEntType.MAP, setter: (data, value) => data._onCard = asMapOfType<String>(value), getter: (data) => data._onCard),

      NUIEntField<NUIColorTheme>(name: "gradient", type: NUIEntType.MAP, setter: (data, value) => data._gradient = asMapOfObject<NUIColorGradient>(NUIColorGradientEnt(), value), getter: (data) => data._gradient),

      NUIEntField<NUIColorTheme>(name: "textTheme", type: NUIEntType.OBJECT, setter: (data, value) => data._textTheme = asObject<NUITextTheme>(NUITextThemeEnt(), value), getter: (data) => data._textTheme),
      NUIEntField<NUIColorTheme>(name: "isDark", type: NUIEntType.BOOLEAN, setter: (data, value) => data._isDark = value, getter: (data) => data._isDark),
    ];
  }

  @override
  NUIColorTheme newInstance() => NUIColorTheme.empty();
}


class NUIColorGradient extends NUIEnt<NUIColorGradient> with NUIColorCodes{
  NUIColorGradient.empty();
  List<String> colors;
  String orientation;

  NUILinearGradient getNUILinearGradient(NUIColorCodes colorCodes){
    setColorCodes(colorCodes.colorCodes);
    return NUILinearGradient(
        colors: colors.map((e) {
          String targetColor = e;
          if(!isPossibleColor(e)){
            targetColor = hexColorFromCode(e) ?? e;
          }
          return HexColor(targetColor);
        }).toList(),
        orientation: gradientOrientationFromValue(orientation)
    );
  }

  @override
  NUIEntMapper<NUIColorGradient> mapper() => NUIColorGradientEnt();
}

class NUIColorGradientEnt extends NUIEntMapper<NUIColorGradient>{
  @override
  String entityName() => "NUIColorGradient";

  @override
  List<NUIEntField> fields() {
    return [
      NUIEntField<NUIColorGradient>(name: "colors", type: NUIEntType.STRING, setter: (data, value) => data.colors = asList<String>(value), getter: (data) => data.colors),
      NUIEntField<NUIColorGradient>(name: "orientation", type: NUIEntType.STRING, setter: (data, value) => data.orientation = value, getter: (data) => data.orientation),
    ];
  }

  @override
  NUIColorGradient newInstance() => NUIColorGradient.empty();
}