import 'package:flutter/material.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_core/data/nui_ent.dart';
import 'package:nui_core/model/hex_color.dart';
import 'package:nui_core/nui_core.dart';
import 'package:nui_theme/nui_theme.dart';

class NUITextStyle extends NUIEnt<NUITextStyle>{
  NUITextStyle.empty();
  NUITextStyle({@required String color, double size: 14, String font, bool bold: false, bool underline: false, bool italic: false}){
    _color = color;
    _size = size;
    _font = font;
    _bold = bold;
    _underline = underline;
    _italic = italic;
  }

  String _color; //Hex color
  double _size;
  String _font; //Based on fonts on asset (Bold, Bold700)
  bool _bold;
  bool _underline;
  bool _italic;

  Color get color => HexColor(_color ?? "#000000");
  double get size => _size ?? 14.0;
  String get font => _font;
  bool get bold => _bold ?? false;
  bool get underline => _underline ?? false;
  bool get italic => _italic ?? false;

  TextStyle get style{
    return TextStyle(fontSize: size, color: color, fontWeight: bold ? FontWeight.bold : FontWeight.normal, decoration: underline ? TextDecoration.underline : TextDecoration.none, fontFamily: font, fontStyle: italic ? FontStyle.italic : FontStyle.normal);
  }

  TextStyle customStyle({BuildContext context, Shadow shadow}){
    try {
      return getFont(context, size: size, singleColor: color, isBold: bold, fontName: font, decoration: underline ? TextDecoration.underline : TextDecoration.none, isItalic: italic);
    }catch(e){
      return TextStyle(fontSize: size, color: color, fontWeight: bold ? FontWeight.bold : FontWeight.normal, shadows: [shadow], decoration: underline ? TextDecoration.underline : TextDecoration.none, fontFamily: font, fontStyle: italic ? FontStyle.italic : FontStyle.normal);
    }
  }

  @override
  NUIEntMapper<NUITextStyle> mapper() => NUITextStyleEnt();
}

class NUITextStyleEnt extends NUIEntMapper<NUITextStyle>{
  @override
  String entityName() => "NUITextStyle";

  @override
  List<NUIEntField> fields() {
    return [
      NUIEntField<NUITextStyle>(name: "color", type: NUIEntType.STRING, setter: (data, value) => data._color = value, getter: (data) => data._color),
      NUIEntField<NUITextStyle>(name: "size", type: NUIEntType.DOUBLE, setter: (data, value) => data._size = value, getter: (data) => data._size),
      NUIEntField<NUITextStyle>(name: "font", type: NUIEntType.STRING, setter: (data, value) => data._font = value, getter: (data) => data._font),
      NUIEntField<NUITextStyle>(name: "bold", type: NUIEntType.BOOLEAN, setter: (data, value) => data._bold = value, getter: (data) => data._bold),
      NUIEntField<NUITextStyle>(name: "underline", type: NUIEntType.BOOLEAN, setter: (data, value) => data._underline = value, getter: (data) => data._underline),
      NUIEntField<NUITextStyle>(name: "italic", type: NUIEntType.BOOLEAN, setter: (data, value) => data._italic = value, getter: (data) => data._italic),
    ];
  }

  @override
  NUITextStyle newInstance() => NUITextStyle.empty();
}


class NUITextTheme extends NUIEnt<NUITextTheme> with NUIColorCodes{
  NUITextTheme.empty();

  String _fontFamily;
  Map<String, NUITextStyle> _header1;
  Map<String, NUITextStyle> _header2;
  Map<String, NUITextStyle> _header3;
  Map<String, NUITextStyle> _header4;
  Map<String, NUITextStyle> _header5;
  Map<String, NUITextStyle> _header6;

  Map<String, NUITextStyle> _subtitle1;
  Map<String, NUITextStyle> _subtitle2;
  Map<String, NUITextStyle> _subtitle3;
  Map<String, NUITextStyle> _subtitle4;
  Map<String, NUITextStyle> _subtitle5;
  Map<String, NUITextStyle> _subtitle6;

  Map<String, NUITextStyle> _body1;
  Map<String, NUITextStyle> _body2;
  Map<String, NUITextStyle> _body3;
  Map<String, NUITextStyle> _body4;
  Map<String, NUITextStyle> _body5;
  Map<String, NUITextStyle> _body6;

  Map<String, NUITextStyle> _button1;
  Map<String, NUITextStyle> _button2;
  Map<String, NUITextStyle> _button3;
  Map<String, NUITextStyle> _button4;
  Map<String, NUITextStyle> _button5;
  Map<String, NUITextStyle> _button6;

  Map<String, NUITextStyle> _caption1;
  Map<String, NUITextStyle> _caption2;
  Map<String, NUITextStyle> _caption3;
  Map<String, NUITextStyle> _caption4;
  Map<String, NUITextStyle> _caption5;
  Map<String, NUITextStyle> _caption6;

  Map<String, NUITextStyle> _overline1;
  Map<String, NUITextStyle> _overline2;
  Map<String, NUITextStyle> _overline3;
  Map<String, NUITextStyle> _overline4;
  Map<String, NUITextStyle> _overline5;
  Map<String, NUITextStyle> _overline6;

  String get fontFamily => _fontFamily;

  NUITextStyle header1({String v}) => _getStyleFromMap(_header1, v);
  NUITextStyle header2({String v}) => _getStyleFromMap(_header2, v);
  NUITextStyle header3({String v}) => _getStyleFromMap(_header3, v);
  NUITextStyle header4({String v}) => _getStyleFromMap(_header4, v);
  NUITextStyle header5({String v}) => _getStyleFromMap(_header5, v);
  NUITextStyle header6({String v}) => _getStyleFromMap(_header6, v);

  NUITextStyle subtitle1({String v}) => _getStyleFromMap(_subtitle1, v);
  NUITextStyle subtitle2({String v}) => _getStyleFromMap(_subtitle2, v);
  NUITextStyle subtitle3({String v}) => _getStyleFromMap(_subtitle3, v);
  NUITextStyle subtitle4({String v}) => _getStyleFromMap(_subtitle4, v);
  NUITextStyle subtitle5({String v}) => _getStyleFromMap(_subtitle5, v);
  NUITextStyle subtitle6({String v}) => _getStyleFromMap(_subtitle6, v);

  NUITextStyle body1({String v}) => _getStyleFromMap(_body1, v);
  NUITextStyle body2({String v}) => _getStyleFromMap(_body2, v);
  NUITextStyle body3({String v}) => _getStyleFromMap(_body3, v);
  NUITextStyle body4({String v}) => _getStyleFromMap(_body4, v);
  NUITextStyle body5({String v}) => _getStyleFromMap(_body5, v);
  NUITextStyle body6({String v}) => _getStyleFromMap(_body6, v);

  NUITextStyle button1({String v}) => _getStyleFromMap(_button1, v);
  NUITextStyle button2({String v}) => _getStyleFromMap(_button2, v);
  NUITextStyle button3({String v}) => _getStyleFromMap(_button3, v);
  NUITextStyle button4({String v}) => _getStyleFromMap(_button4, v);
  NUITextStyle button5({String v}) => _getStyleFromMap(_button5, v);
  NUITextStyle button6({String v}) => _getStyleFromMap(_button6, v);

  NUITextStyle caption1({String v}) => _getStyleFromMap(_caption1, v);
  NUITextStyle caption2({String v}) => _getStyleFromMap(_caption2, v);
  NUITextStyle caption3({String v}) => _getStyleFromMap(_caption3, v);
  NUITextStyle caption4({String v}) => _getStyleFromMap(_caption4, v);
  NUITextStyle caption5({String v}) => _getStyleFromMap(_caption5, v);
  NUITextStyle caption6({String v}) => _getStyleFromMap(_caption6, v);

  NUITextStyle overline1({String v}) => _getStyleFromMap(_overline1, v);
  NUITextStyle overline2({String v}) => _getStyleFromMap(_overline2, v);
  NUITextStyle overline3({String v}) => _getStyleFromMap(_overline3, v);
  NUITextStyle overline4({String v}) => _getStyleFromMap(_overline4, v);
  NUITextStyle overline5({String v}) => _getStyleFromMap(_overline5, v);
  NUITextStyle overline6({String v}) => _getStyleFromMap(_overline6, v);

  TextTheme getTextTheme(){
    return TextTheme(
      headline1: header1().style,
      headline2: header2().style,
      headline3: header3().style,
      headline4: header4().style,
      headline5: header5().style,
      headline6: header6().style,
      subtitle1: subtitle1().style,
      subtitle2: subtitle2().style,
      bodyText1: subtitle3().style,
      bodyText2: subtitle4().style,
      caption: caption1().style,
      button: button1().style,
      overline: overline1().style,
    );
  }

  @override
  NUIEntMapper<NUITextTheme> mapper() => NUITextThemeEnt();

  NUITextStyle _getStyleFromMap(Map<String, NUITextStyle> group, String variant){
    if(group == null || isNullOrEmpty(group.values.toList())){
      throwNUI("NUITextTheme", "You have not configured text group");
    }

    if(isNullOrEmpty(variant)){
      final list = group.values.toList();
      final textStyle = list[0];
      textStyle._font = textStyle.font ?? fontFamily;
      if(!isPossibleColor(textStyle._color)){
        textStyle._color = hexColorFromCode(textStyle._color) ?? textStyle._color;
      }
      return textStyle;
    }

    final textStyle = group[variant];
    if(textStyle == null){
      throwNUI("NUITextTheme", "You have not configured a text style for variant : $variant");
    }
    textStyle._font = textStyle.font ?? fontFamily;
    if(!isPossibleColor(textStyle._color)){
      textStyle._color = hexColorFromCode(textStyle._color) ?? textStyle._color;
    }
    return textStyle;
  }
}

class NUITextThemeEnt extends NUIEntMapper<NUITextTheme>{
  @override
  String entityName() => "NUITextTheme";

  @override
  List<NUIEntField> fields() {
    return [
      NUIEntField<NUITextTheme>(name: "fontFamily", type: NUIEntType.MAP, setter: (data, value) => data._fontFamily = value, getter: (data) => data._fontFamily),

      NUIEntField<NUITextTheme>(name: "header1", type: NUIEntType.MAP, setter: (data, value) => data._header1 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._header1),
      NUIEntField<NUITextTheme>(name: "header2", type: NUIEntType.MAP, setter: (data, value) => data._header2 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._header2),
      NUIEntField<NUITextTheme>(name: "header3", type: NUIEntType.MAP, setter: (data, value) => data._header3 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._header3),
      NUIEntField<NUITextTheme>(name: "header4", type: NUIEntType.MAP, setter: (data, value) => data._header4 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._header4),
      NUIEntField<NUITextTheme>(name: "header5", type: NUIEntType.MAP, setter: (data, value) => data._header5 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._header5),
      NUIEntField<NUITextTheme>(name: "header6", type: NUIEntType.MAP, setter: (data, value) => data._header6 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._header6),

      NUIEntField<NUITextTheme>(name: "subtitle1", type: NUIEntType.MAP, setter: (data, value) => data._subtitle1 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._subtitle1),
      NUIEntField<NUITextTheme>(name: "subtitle2", type: NUIEntType.MAP, setter: (data, value) => data._subtitle2 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._subtitle2),
      NUIEntField<NUITextTheme>(name: "subtitle3", type: NUIEntType.MAP, setter: (data, value) => data._subtitle3 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._subtitle3),
      NUIEntField<NUITextTheme>(name: "subtitle4", type: NUIEntType.MAP, setter: (data, value) => data._subtitle4 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._subtitle4),
      NUIEntField<NUITextTheme>(name: "subtitle5", type: NUIEntType.MAP, setter: (data, value) => data._subtitle5 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._subtitle5),
      NUIEntField<NUITextTheme>(name: "subtitle6", type: NUIEntType.MAP, setter: (data, value) => data._subtitle6 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._subtitle6),

      NUIEntField<NUITextTheme>(name: "body1", type: NUIEntType.MAP, setter: (data, value) => data._body1 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._body1),
      NUIEntField<NUITextTheme>(name: "body2", type: NUIEntType.MAP, setter: (data, value) => data._body2 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._body2),
      NUIEntField<NUITextTheme>(name: "body3", type: NUIEntType.MAP, setter: (data, value) => data._body3 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._body3),
      NUIEntField<NUITextTheme>(name: "body4", type: NUIEntType.MAP, setter: (data, value) => data._body4 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._body4),
      NUIEntField<NUITextTheme>(name: "body5", type: NUIEntType.MAP, setter: (data, value) => data._body5 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._body5),
      NUIEntField<NUITextTheme>(name: "body6", type: NUIEntType.MAP, setter: (data, value) => data._body6 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._body6),

      NUIEntField<NUITextTheme>(name: "button1", type: NUIEntType.MAP, setter: (data, value) => data._button1 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._button1),
      NUIEntField<NUITextTheme>(name: "button2", type: NUIEntType.MAP, setter: (data, value) => data._button2 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._button2),
      NUIEntField<NUITextTheme>(name: "button3", type: NUIEntType.MAP, setter: (data, value) => data._button3 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._button3),
      NUIEntField<NUITextTheme>(name: "button4", type: NUIEntType.MAP, setter: (data, value) => data._button4 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._button4),
      NUIEntField<NUITextTheme>(name: "button5", type: NUIEntType.MAP, setter: (data, value) => data._button5 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._button5),
      NUIEntField<NUITextTheme>(name: "button6", type: NUIEntType.MAP, setter: (data, value) => data._button6 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._button6),

      NUIEntField<NUITextTheme>(name: "caption1", type: NUIEntType.MAP, setter: (data, value) => data._caption1 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._caption1),
      NUIEntField<NUITextTheme>(name: "caption2", type: NUIEntType.MAP, setter: (data, value) => data._caption2 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._caption2),
      NUIEntField<NUITextTheme>(name: "caption3", type: NUIEntType.MAP, setter: (data, value) => data._caption3 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._caption3),
      NUIEntField<NUITextTheme>(name: "caption4", type: NUIEntType.MAP, setter: (data, value) => data._caption4 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._caption4),
      NUIEntField<NUITextTheme>(name: "caption5", type: NUIEntType.MAP, setter: (data, value) => data._caption5 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._caption5),
      NUIEntField<NUITextTheme>(name: "caption6", type: NUIEntType.MAP, setter: (data, value) => data._caption6 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._caption6),

      NUIEntField<NUITextTheme>(name: "overline1", type: NUIEntType.MAP, setter: (data, value) => data._overline1 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._overline1),
      NUIEntField<NUITextTheme>(name: "overline2", type: NUIEntType.MAP, setter: (data, value) => data._overline2 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._overline2),
      NUIEntField<NUITextTheme>(name: "overline3", type: NUIEntType.MAP, setter: (data, value) => data._overline3 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._overline3),
      NUIEntField<NUITextTheme>(name: "overline4", type: NUIEntType.MAP, setter: (data, value) => data._overline4 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._overline4),
      NUIEntField<NUITextTheme>(name: "overline5", type: NUIEntType.MAP, setter: (data, value) => data._overline5 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._overline5),
      NUIEntField<NUITextTheme>(name: "overline6", type: NUIEntType.MAP, setter: (data, value) => data._overline6 = asMapOfObject<NUITextStyle>(NUITextStyleEnt(), value), getter: (data) => data._overline6),
    ];
  }

  @override
  NUITextTheme newInstance() => NUITextTheme.empty();
}