import 'package:flutter/cupertino.dart';
import 'package:nui_core/nui_core.dart';
import 'nui_color_theme.dart';

class NUIColorCodes{
  Map<String, String> _colorCodes;

  void setColorCodes(Map<String, String> colorCodes){
    if(_colorCodes == null || _colorCodes.isEmpty) {
      _colorCodes = colorCodes;
    }
  }

  Map<String, String> get colorCodes => _colorCodes;

  Color colorFromCode(String code){
    if(_colorCodes == null || _colorCodes.isEmpty) return null;
    final targetColor = _colorCodes[code];
    if(targetColor != null) {
      return HexColor(targetColor);
    }
    return null;
  }

  String hexColorFromCode(String code){
    if(_colorCodes == null || _colorCodes.isEmpty) return null;
    final targetColor = _colorCodes[code];
    return targetColor;
  }

  bool isPossibleColor(String color){
    if(color.startsWith("#")) return true;
    return false;
  }
}

class NUIThemeConfig extends NUIEnt<NUIThemeConfig> with NUIColorCodes{
  NUIThemeConfig.empty();
  NUIColorTheme _lightTheme;
  NUIColorTheme _darkTheme;
  bool _startWithDark;

  NUIColorTheme get lightTheme => _lightTheme;
  NUIColorTheme get darkTheme => _darkTheme;
  bool get isStartWithDark => _startWithDark ?? false;

  @override
  NUIEntMapper<NUIThemeConfig> mapper() => NUIThemeConfigEnt();
}

class NUIThemeConfigEnt extends NUIEntMapper<NUIThemeConfig>{
  @override
  String entityName() => "NUIThemeConfig";

  @override
  List<NUIEntField> fields() {
    return [
      NUIEntField<NUIThemeConfig>(name: "colorCodes", type: NUIEntType.MAP, setter: (data, value) => data._colorCodes = asMapOfType<String>(value), getter: (data) => data._colorCodes),
      NUIEntField<NUIThemeConfig>(name: "lightTheme", type: NUIEntType.OBJECT, setter: (data, value) => data._lightTheme = asObject<NUIColorTheme>(NUIColorThemeEnt(), value), getter: (data) => data._lightTheme),
      NUIEntField<NUIThemeConfig>(name: "darkTheme", type: NUIEntType.OBJECT, setter: (data, value) => data._darkTheme = asObject<NUIColorTheme>(NUIColorThemeEnt(), value), getter: (data) => data._darkTheme),
      NUIEntField<NUIThemeConfig>(name: "startWithDark", type: NUIEntType.BOOLEAN, setter: (data, value) => data._startWithDark = value, getter: (data) => data._startWithDark),
    ];
  }

  @override
  NUIThemeConfig newInstance() => NUIThemeConfig.empty();
}